#ifndef QUICKTIME_H
#define QUICKTIME_H

#ifdef __cplusplus
extern "C" {
#endif
#include "mpeg4ip.h"

#include <stdio.h>
#include <stdlib.h>


#include "rtphint.h"
#include "private.h"
#include "funcprotos.h"
#include "sizes.h"


/* This is the reference for all your library entry points. */

/* Color models.  Not all compression formats support QUICKTIME_BGR8880. */
/* You must verify the support of QUICKTIME_BGR8880 with quicktime_test_colormodel. */
#define QUICKTIME_RGB888   0
#define QUICKTIME_BGR8880  1
#define QUICKTIME_RGBA8880 2

/* ===================== compression formats =================== // */
/* The compression formats are the formats supported natively by Quicktime 4 Linux. */

/* DV works only for decoding. */
#define QUICKTIME_DV "dvc "

/* RGB uncompressed.  Allows alpha */
#define QUICKTIME_RAW  "raw "

/* Jpeg Photo */
#define QUICKTIME_JPEG "jpeg"

/* Motion JPEG-A. */
#define QUICKTIME_MJPA "mjpa"

/* YUV9.  What on earth for? */
#define QUICKTIME_YUV9 "YVU9"

/* RTjpeg, proprietary but fast? */
#define QUICKTIME_RTJ0 "RTJ0"

/* YUV 4:2:2 */
#define QUICKTIME_YUV2 "yuv2"

/* YUV 4:2:0  NOT COMPATIBLE WITH STANDARD QUICKTIME */
#define QUICKTIME_YUV4 "yuv4"

/* Concatenated png images.  Allows alpha */
#define QUICKTIME_PNG "png "

/* Audio formats */

/* Unsigned 8 bit */
#ifndef QUICKTIME_RAW
#define QUICKTIME_RAW "raw "
#endif

/* IMA4 */
#define QUICKTIME_IMA4 "ima4"

/* Twos compliment 8, 16, 24 */
#define QUICKTIME_TWOS "twos"

/* ulaw */
#define QUICKTIME_ULAW "ulaw"

/* =========================== public interface ========================= // */

/* return 1 if the file is a quicktime file */
int quicktime_check_sig(const char *path);

/* call this first to open the file and create all the objects */
quicktime_t* quicktime_open(char *filename, int rd, int wr, int append);

/* make the quicktime file streamable */
int quicktime_make_streamable(char *in_path, char *out_path);

/* Set various options in the file. */
int quicktime_set_time_scale(quicktime_t *file, int time_scale);
int quicktime_set_copyright(quicktime_t *file, char *string);
int quicktime_set_name(quicktime_t *file, char *string);
int quicktime_set_info(quicktime_t *file, char *string);
int quicktime_get_time_scale(quicktime_t *file);
char* quicktime_get_copyright(quicktime_t *file);
char* quicktime_get_name(quicktime_t *file);
char* quicktime_get_info(quicktime_t *file);

/* Read all the information about the file. */
/* Requires a MOOV atom be present in the file. */
/* If no MOOV atom exists return 1 else return 0. */
int quicktime_read_info(quicktime_t *file);

/* set up tracks in a new file after opening and before writing */
/* returns the number of quicktime tracks allocated */
/* audio is stored two channels per quicktime track */
int quicktime_set_audio(quicktime_t *file, int channels, long sample_rate, int bits, int sample_size, int time_scale, int sample_duration, char *compressor);

int quicktime_set_audio_hint(quicktime_t *file, int track, char *payloadName, u_int* pPayloadNumber, int maxPktSize);

/* Samplerate can be set after file is created */
int quicktime_set_framerate(quicktime_t *file, float framerate);

/* video is stored one layer per quicktime track */
int quicktime_set_video(quicktime_t *file, int tracks, int frame_w, int frame_h, float frame_rate, int time_scale, char *compressor);

int quicktime_set_video_hint(quicktime_t *file, int track, char *payloadName, u_int* pPayloadNumber, int maxPktSize);

/* routines for setting various video parameters */
/* should be called after set_video */
int quicktime_set_jpeg(quicktime_t *file, int quality, int use_float);

/* Set the depth of the track. */
int quicktime_set_depth(quicktime_t *file, int depth, int track);

/* close the file and delete all the objects */
int quicktime_write(quicktime_t *file);
int quicktime_destroy(quicktime_t *file);
int quicktime_close(quicktime_t *file);

/* get length information */
/* channel numbers start on 1 for audio and video */
long quicktime_audio_length(quicktime_t *file, int track);
long quicktime_video_length(quicktime_t *file, int track);

/* get position information */
long quicktime_audio_position(quicktime_t *file, int track);
long quicktime_video_position(quicktime_t *file, int track);

/* get file information */
int quicktime_video_tracks(quicktime_t *file);
int quicktime_audio_tracks(quicktime_t *file);

int quicktime_has_audio(quicktime_t *file);
long quicktime_audio_sample_rate(quicktime_t *file, int track);
int quicktime_audio_bits(quicktime_t *file, int track);
int quicktime_track_channels(quicktime_t *file, int track);
int quicktime_audio_time_scale(quicktime_t *file, int track);
int quicktime_audio_sample_duration(quicktime_t *file, int track);
char* quicktime_audio_compressor(quicktime_t *file, int track);

int quicktime_has_video(quicktime_t *file);
int quicktime_video_width(quicktime_t *file, int track);
int quicktime_video_height(quicktime_t *file, int track);
int quicktime_video_depth(quicktime_t *file, int track);
float quicktime_video_frame_rate(quicktime_t *file, int track);
char* quicktime_video_compressor(quicktime_t *file, int track);
int quicktime_video_time_scale(quicktime_t *file, int track);

int quicktime_video_frame_time(quicktime_t *file, int track, long frame,
	long *start_time, int *duration);

int quicktime_get_iod_audio_profile_level(quicktime_t *file);
int quicktime_set_iod_audio_profile_level(quicktime_t *file, int id);
int quicktime_get_iod_video_profile_level(quicktime_t *file);
int quicktime_set_iod_video_profile_level(quicktime_t *file, int id);

int quicktime_get_mp4_video_decoder_config(quicktime_t *file, int track, u_char** ppBuf, int* pBufSize);
int quicktime_set_mp4_video_decoder_config(quicktime_t *file, int track, u_char* pBuf, int bufSize);
int quicktime_get_mp4_audio_decoder_config(quicktime_t *file, int track, u_char** ppBuf, int* pBufSize);
int quicktime_set_mp4_audio_decoder_config(quicktime_t *file, int track, u_char* pBuf, int bufSize);

char* quicktime_get_session_sdp(quicktime_t *file);
int quicktime_set_session_sdp(quicktime_t *file, char* sdpString);

int quicktime_add_audio_sdp(quicktime_t *file, char* sdpString, int track, int hintTrack);

int quicktime_add_video_sdp(quicktime_t *file, char* sdpString, int track, int hintTrack);

int quicktime_set_audio_hint_max_rate(quicktime_t *file, int granularity, int maxBitRate, int audioTrack, int hintTrack);

int quicktime_set_video_hint_max_rate(quicktime_t *file, int granularity, int maxBitRate, int videoTrack, int hintTrack);

/* number of bytes of raw data in this frame */
long quicktime_frame_size(quicktime_t *file, long frame, int track);

/* get the quicktime track and channel that the audio channel belongs to */
/* channels and tracks start on 0 */
int quicktime_channel_location(quicktime_t *file, int *quicktime_track, int *quicktime_channel, int channel);

/* file positioning */
int quicktime_seek_end(quicktime_t *file);
int quicktime_seek_start(quicktime_t *file);

/* set position of file descriptor relative to a track */
int quicktime_set_audio_position(quicktime_t *file, long sample, int track);
int quicktime_set_video_position(quicktime_t *file, long frame, int track);

/* ========================== Access to raw data follows. */
/* write data for one quicktime track */
/* the user must handle conversion to the channels in this track */
int quicktime_write_audio(quicktime_t *file, char *audio_buffer, long samples, int track);
int quicktime_write_audio_frame(quicktime_t *file, unsigned char *audio_buffer, long bytes, int track);
int quicktime_write_audio_hint(quicktime_t *file, unsigned char *hintBuffer, 
	long hintBufferSize, int audioTrack, int hintTrack, long hintSampleDuration);

int quicktime_write_video_frame(quicktime_t *file, unsigned char *video_buffer, long bytes, int track, u_char isKeyFrame, long duration, long renderingOffset);
int quicktime_write_video_hint(quicktime_t *file, unsigned char *hint_buffer, long hintBufferSize, int videoTrack, int hintTrack, long hintSampleDuration, u_char isKeyFrame);

/* for writing a frame using a library that needs a file descriptor */
int quicktime_write_frame_init(quicktime_t *file, int track); /* call before fwrite */
FILE* quicktime_get_fd(quicktime_t *file);     /* return a file descriptor */
int quicktime_write_frame_end(quicktime_t *file, int track); /* call after fwrite */

/* For reading and writing audio to a file descriptor. */
int quicktime_write_audio_end(quicktime_t *file, int track, long samples); /* call after fwrite */

/* Read an entire chunk. */
/* read the number of bytes starting at the byte_start in the specified chunk */
/* You must provide enough space to store the chunk. */
int quicktime_read_chunk(quicktime_t *file, char *output, int track, long chunk, long byte_start, long byte_len);

/* read raw data */
long quicktime_read_audio(quicktime_t *file, char *audio_buffer, long samples, int track);
long quicktime_read_audio_frame(quicktime_t *file, unsigned char *audio_buffer, int maxBytes, int track);
long quicktime_read_frame(quicktime_t *file, unsigned char *video_buffer, int track);

/* for reading frame using a library that needs a file descriptor */
/* Frame caching doesn't work here. */
int quicktime_read_frame_init(quicktime_t *file, int track);
int quicktime_read_frame_end(quicktime_t *file, int track);

/* for RTP hints */
void quicktime_init_hint_sample(u_char* hintBuf, u_int* hintBufSize);
void quicktime_add_hint_packet(u_char* hintBuf, u_int* hintBufSize,
					u_int payloadNumber, u_int rtpSeqNum);
void quicktime_add_hint_immed_data(u_char* hintBuf, u_int* hintBufSize, 
					u_char* data, u_int length);
void quicktime_add_hint_sample_data(u_char* hintBuf, u_int* hintBufSize, 
					u_int fromSampleNum, u_int offset, u_int length);

void quicktime_set_hint_timestamp_offset(u_char* hintBuf, u_int* hintBufSize, int offset);
void quicktime_set_hint_Mbit(u_char* hintBuf);
void quicktime_set_hint_Bframe(u_char* hintBuf);
void quicktime_set_hint_repeat(u_char* hintBuf);
void quicktime_set_hint_repeat_offset(u_char* hintBuf, u_int32_t offset);

int quicktime_dump_hint_sample(u_char* hintBuf);
int quicktime_dump_hint_packet(u_char* hintBuf);
int quicktime_dump_hint_data(u_char* hintBuf);

int quicktime_get_hint_size(u_char* hintBuf);
int quicktime_get_hint_info(u_char* hintBuf, u_int hintBufSize, quicktime_hint_info_t* pHintInfo);

/* ===================== Access to built in codecs follows. */

/* If the codec for this track is supported in the library return 1. */
int quicktime_supported_video(quicktime_t *file, int track);
int quicktime_supported_audio(quicktime_t *file, int track);

/* The codec can generate the color model */
int quicktime_test_colormodel(quicktime_t *file, 
		int colormodel, 
		int track);

/* Decode or encode the frame into a frame buffer. */
/* All the frame buffers passed to these functions are unsigned char */
/* rows with 3 bytes per pixel.  The byte order per 3 byte pixel is */
/* RGB. */
int quicktime_decode_video(quicktime_t *file, unsigned char **row_pointers, int track);
int quicktime_encode_video(quicktime_t *file, unsigned char **row_pointers, int track);
long quicktime_decode_scaled(quicktime_t *file, 
	int in_x,                    /* Location of input frame to take picture */
	int in_y,
	int in_w,
	int in_h,
	int out_w,                   /* Dimensions of output frame */
	int out_h,
	int color_model,             /* One of the color models defined above */
	unsigned char **row_pointers, 
	int track);

/* Decode or encode audio for a single channel into the buffer. */
/* Pass a buffer for the _i or the _f argument if you want int16 or float data. */
/* Notice that encoding requires an array of pointers to each channel. */
int quicktime_decode_audio(quicktime_t *file, QUICKTIME_INT16 *output_i, float *output_f, long samples, int channel);
int quicktime_encode_audio(quicktime_t *file, QUICKTIME_INT16 **input_i, float **input_f, long samples);

/* Dump the file structures for the currently opened file. */
int quicktime_dump(quicktime_t *file);

/* Specify the number of cpus to utilize. */
int quicktime_set_cpus(quicktime_t *file, int cpus);

/* Specify whether to read contiguously or not. */
/* preload is the number of bytes to read ahead. */
int quicktime_set_preload(quicktime_t *file, long preload);

/* Test the 32 bit overflow */
int quicktime_test_position(quicktime_t *file);

int quicktime_atom_is(quicktime_atom_t *atom, char *type);
int quicktime_atom_read_header(quicktime_t *file, quicktime_atom_t *atom);
int quicktime_atom_skip(quicktime_t *file, quicktime_atom_t *atom);
int quicktime_chunk_of_sample(long *chunk_sample, long *chunk, quicktime_trak_t *trak, long sample);
int quicktime_ctab_delete(quicktime_ctab_t *ctab);
int quicktime_ctab_dump(quicktime_ctab_t *ctab);
int quicktime_ctts_delete(quicktime_ctts_t *ctts);
int quicktime_ctts_dump(quicktime_ctts_t *ctts);
int quicktime_ctts_init(quicktime_ctts_t *ctts);
int quicktime_ctts_init_common(quicktime_t *file, quicktime_ctts_t *ctts);
int quicktime_delete_audio_map(quicktime_audio_map_t *atrack);
int quicktime_delete_video_map(quicktime_video_map_t *vtrack);
int quicktime_dimm_delete(quicktime_dimm_t *dimm);
int quicktime_dimm_dump(quicktime_dimm_t *dimm);
int quicktime_dimm_init(quicktime_dimm_t *dimm);
int quicktime_dinf_delete(quicktime_dinf_t *dinf);
int quicktime_dinf_dump(quicktime_dinf_t *dinf);
int quicktime_dmax_delete(quicktime_dmax_t *dmax);
int quicktime_dmax_dump(quicktime_dmax_t *dmax);
int quicktime_dmax_init(quicktime_dmax_t *dmax);
int quicktime_dmed_delete(quicktime_dmed_t *dmed);
int quicktime_dmed_dump(quicktime_dmed_t *dmed);
int quicktime_dmed_init(quicktime_dmed_t *dmed);
int quicktime_dref_delete(quicktime_dref_t *dref);
int quicktime_dref_dump(quicktime_dref_t *dref);
int quicktime_drep_delete(quicktime_drep_t *drep);
int quicktime_drep_dump(quicktime_drep_t *drep);
int quicktime_drep_init(quicktime_drep_t *drep);
int quicktime_dump_hint_tlv(u_char* hintBuf);
int quicktime_elst_delete(quicktime_elst_t *elst);
int quicktime_elst_dump(quicktime_elst_t *elst);
int quicktime_esds_delete(quicktime_esds_t *esds);
int quicktime_esds_dump(quicktime_esds_t *esds);
int quicktime_esds_get_decoder_config(quicktime_esds_t* esds, u_char** ppBuf, int* pBufSize);
int quicktime_esds_init(quicktime_esds_t *esds);
int quicktime_esds_set_decoder_config(quicktime_esds_t* esds, u_char* pBuf, int bufSize);
int quicktime_get_packet_size(u_char* hintBuf);
int quicktime_gmhd_delete(quicktime_gmhd_t *gmhd);
int quicktime_gmhd_dump(quicktime_gmhd_t *gmhd);
int quicktime_gmhd_init(quicktime_gmhd_t *gmhd);
int quicktime_gmin_delete(quicktime_gmin_t *gmin);
int quicktime_gmin_dump(quicktime_gmin_t *gmin);
int quicktime_gmin_init(quicktime_gmin_t *gmin);
int quicktime_hdlr_delete(quicktime_hdlr_t *hdlr);
int quicktime_hdlr_dump(quicktime_hdlr_t *hdlr);
int quicktime_hdlr_init_hint(quicktime_hdlr_t *hdlr);
int quicktime_hinf_delete(quicktime_hinf_t *hinf);
int quicktime_hinf_dump(quicktime_hinf_t *hinf);
int quicktime_hinf_init(quicktime_hinf_t *hinf);
int quicktime_hint_hnti_delete(quicktime_hint_hnti_t *hnti);
int quicktime_hint_hnti_dump(quicktime_hint_hnti_t *hnti);
int quicktime_hint_hnti_init(quicktime_hint_hnti_t *hnti);
int quicktime_hmhd_delete(quicktime_hmhd_t *hmhd);
int quicktime_hmhd_dump(quicktime_hmhd_t *hmhd);
int quicktime_hmhd_init(quicktime_hmhd_t *hmhd);
int quicktime_init_audio_map(quicktime_audio_map_t *atrack, quicktime_trak_t *trak);
int quicktime_init_video_map(quicktime_video_map_t *vtrack, quicktime_trak_t *trak);
int quicktime_iods_delete(quicktime_iods_t *iods);
int quicktime_iods_dump(quicktime_iods_t *iods);
int quicktime_iods_init(quicktime_iods_t *iods);
int quicktime_iods_set_audio_profile(quicktime_iods_t* iods, int id);
int quicktime_iods_set_video_profile(quicktime_iods_t* iods, int id);
int quicktime_match_32(char *input, char *output);
int quicktime_matrix_dump(quicktime_matrix_t *matrix);
int quicktime_maxr_delete(quicktime_maxr_t *maxr);
int quicktime_maxr_dump(quicktime_maxr_t *maxr);
int quicktime_maxr_init(quicktime_maxr_t *maxr);
int quicktime_mdat_delete(quicktime_mdat_t *mdat);
int quicktime_mdhd_delete(quicktime_mdhd_t *mdhd);
int quicktime_mdhd_dump(quicktime_mdhd_t *mdhd);
int quicktime_mdhd_init_hint(quicktime_t *file, quicktime_mdhd_t *mdhd, quicktime_trak_t *refTrak, int time_scale);
int quicktime_minf_delete(quicktime_minf_t *minf);
int quicktime_minf_dump(quicktime_minf_t *minf);
int quicktime_mjht_delete(quicktime_mjht_t *mjht);
int quicktime_mjht_dump(quicktime_mjht_t *mjht);
int quicktime_mjqt_delete(quicktime_mjqt_t *mjqt);
int quicktime_mjqt_dump(quicktime_mjqt_t *mjqt);
int quicktime_moov_delete(quicktime_moov_t *moov);
int quicktime_moov_dump(quicktime_moov_t *moov);
int quicktime_mvhd_delete(quicktime_mvhd_t *mvhd);
int quicktime_mvhd_dump(quicktime_mvhd_t *mvhd);
int quicktime_nump_delete(quicktime_nump_t *nump);
int quicktime_nump_dump(quicktime_nump_t *nump);
int quicktime_nump_init(quicktime_nump_t *nump);
int quicktime_payt_delete(quicktime_payt_t *payt);
int quicktime_payt_dump(quicktime_payt_t *payt);
int quicktime_payt_init(quicktime_payt_t *payt);
int quicktime_pmax_delete(quicktime_pmax_t *pmax);
int quicktime_pmax_dump(quicktime_pmax_t *pmax);
int quicktime_pmax_init(quicktime_pmax_t *pmax);
int quicktime_print_chars(char *desc, char *input, int len);
int quicktime_read_ctab(quicktime_t *file, quicktime_ctab_t *ctab);
int quicktime_read_ctts(quicktime_t *file, quicktime_ctts_t *ctts);
int quicktime_read_dimm(quicktime_t *file, quicktime_dimm_t *dimm);
int quicktime_read_dinf(quicktime_t *file, quicktime_dinf_t *dinf, quicktime_atom_t *dinf_atom);
int quicktime_read_dmax(quicktime_t *file, quicktime_dmax_t *dmax);
int quicktime_read_dmed(quicktime_t *file, quicktime_dmed_t *dmed);
int quicktime_read_dref(quicktime_t *file, quicktime_dref_t *dref);
int quicktime_read_drep(quicktime_t *file, quicktime_drep_t *drep);
int quicktime_read_elst(quicktime_t *file, quicktime_elst_t *elst);
int quicktime_read_esds(quicktime_t *file, quicktime_esds_t *esds);
int quicktime_read_gmhd(quicktime_t *file, quicktime_gmhd_t *gmhd, quicktime_atom_t *parent_atom);
int quicktime_read_gmin(quicktime_t *file, quicktime_gmin_t *gmin);
int quicktime_read_hdlr(quicktime_t *file, quicktime_hdlr_t *hdlr);
int quicktime_read_hinf(quicktime_t *file, quicktime_hinf_t *hinf, quicktime_atom_t *parent_atom);
int quicktime_read_hint_hnti(quicktime_t *file, quicktime_hint_hnti_t *hnti, quicktime_atom_t *parent_atom);
int quicktime_read_hmhd(quicktime_t *file, quicktime_hmhd_t *hmhd);
int quicktime_read_iods(quicktime_t *file, quicktime_iods_t *iods);
int quicktime_read_matrix(quicktime_t *file, quicktime_matrix_t *matrix);
int quicktime_read_maxr(quicktime_t *file, quicktime_maxr_t *maxr);
int quicktime_read_mdat(quicktime_t *file, quicktime_mdat_t *mdat, quicktime_atom_t *parent_atom);
int quicktime_read_mdhd(quicktime_t *file, quicktime_mdhd_t *mdhd);
int quicktime_read_minf(quicktime_t *file, quicktime_minf_t *minf, quicktime_atom_t *parent_atom);
int quicktime_read_moov(quicktime_t *file, quicktime_moov_t *moov, quicktime_atom_t *parent_atom);
int quicktime_read_mvhd(quicktime_t *file, quicktime_mvhd_t *mvhd);
int quicktime_read_nump(quicktime_t *file, quicktime_nump_t *nump);
int quicktime_read_payt(quicktime_t *file, quicktime_payt_t *payt);
int quicktime_read_pmax(quicktime_t *file, quicktime_pmax_t *pmax);
int quicktime_read_rtp(quicktime_t *file, quicktime_rtp_t *rtp, quicktime_atom_t* rtp_atom);
int quicktime_read_sdp(quicktime_t *file, quicktime_sdp_t *sdp, quicktime_atom_t* sdp_atom);
int quicktime_read_smhd(quicktime_t *file, quicktime_smhd_t *smhd);
int quicktime_read_stbl(quicktime_t *file, quicktime_minf_t *minf, quicktime_stbl_t *stbl, quicktime_atom_t *parent_atom);
int quicktime_read_stco(quicktime_t *file, quicktime_stco_t *stco);
int quicktime_read_stsc(quicktime_t *file, quicktime_stsc_t *stsc);
int quicktime_read_stsd(quicktime_t *file, quicktime_minf_t *minf, quicktime_stsd_t *stsd);
int quicktime_read_stsd_audio(quicktime_t *file, quicktime_stsd_table_t *table, quicktime_atom_t *parent_atom);
int quicktime_read_stsd_hint(quicktime_t *file, quicktime_stsd_table_t *table, quicktime_atom_t *parent_atom);
int quicktime_read_stsd_table(quicktime_t *file, quicktime_minf_t *minf, quicktime_stsd_table_t *table);
int quicktime_read_stsd_video(quicktime_t *file, quicktime_stsd_table_t *table, quicktime_atom_t *parent_atom);
int quicktime_read_stss(quicktime_t *file, quicktime_stss_t *stss);
int quicktime_read_stsz(quicktime_t *file, quicktime_stsz_t *stsz);
int quicktime_read_stts(quicktime_t *file, quicktime_stts_t *stts);
int quicktime_read_tims(quicktime_t *file, quicktime_tims_t *tims);
int quicktime_read_tmax(quicktime_t *file, quicktime_tmax_t *tmax);
int quicktime_read_tmin(quicktime_t *file, quicktime_tmin_t *tmin);
int quicktime_read_tpyl(quicktime_t *file, quicktime_tpyl_t *tpyl);
int quicktime_read_trak(quicktime_t *file, quicktime_trak_t *trak, quicktime_atom_t *trak_atom);
int quicktime_read_trpy(quicktime_t *file, quicktime_trpy_t *trpy);
int quicktime_read_udta(quicktime_t *file, quicktime_udta_t *udta, quicktime_atom_t *udta_atom);
int quicktime_read_vmhd(quicktime_t *file, quicktime_vmhd_t *vmhd);
int quicktime_rtp_delete(quicktime_rtp_t *rtp);
int quicktime_rtp_dump(quicktime_rtp_t *rtp);
int quicktime_rtp_init(quicktime_rtp_t *rtp);
int quicktime_rtp_set(quicktime_rtp_t *rtp, char *string);
int quicktime_sdp_append(quicktime_sdp_t *sdp, char *appendString);
int quicktime_sdp_delete(quicktime_sdp_t *sdp);
int quicktime_sdp_dump(quicktime_sdp_t *sdp);
int quicktime_sdp_init(quicktime_sdp_t *sdp);
int quicktime_sdp_set(quicktime_sdp_t *sdp, char *string);
int quicktime_set_position(quicktime_t *file, long position);
int quicktime_set_udta_string(char **string, int *size, char *new_string);
int quicktime_shift_offsets(quicktime_moov_t *moov, long offset);
int quicktime_smhd_delete(quicktime_smhd_t *smhd);
int quicktime_smhd_dump(quicktime_smhd_t *smhd);
int quicktime_stbl_delete(quicktime_stbl_t *stbl);
int quicktime_stbl_dump(void *minf_ptr, quicktime_stbl_t *stbl);
int quicktime_stco_delete(quicktime_stco_t *stco);
int quicktime_stco_dump(quicktime_stco_t *stco);
int quicktime_stsc_delete(quicktime_stsc_t *stsc);
int quicktime_stsc_dump(quicktime_stsc_t *stsc);
int quicktime_stsd_audio_dump(quicktime_stsd_table_t *table);
int quicktime_stsd_delete(quicktime_stsd_t *stsd);
int quicktime_stsd_dump(void *minf_ptr, quicktime_stsd_t *stsd);
int quicktime_stsd_hint_dump(quicktime_stsd_table_t *table);
int quicktime_stsd_init_hint(quicktime_t *file, quicktime_stsd_t *stsd, int maxPktSize, int timeScale);
int quicktime_stsd_table_delete(quicktime_stsd_table_t *table);
int quicktime_stsd_table_dump(void *minf_ptr, quicktime_stsd_table_t *table);
int quicktime_stsd_video_dump(quicktime_stsd_table_t *table);
int quicktime_stss_delete(quicktime_stss_t *stss);
int quicktime_stss_dump(quicktime_stss_t *stss);
int quicktime_stsz_delete(quicktime_stsz_t *stsz);
int quicktime_stsz_dump(quicktime_stsz_t *stsz);
int quicktime_stts_delete(quicktime_stts_t *stts);
int quicktime_stts_dump(quicktime_stts_t *stts);
int quicktime_tims_init(quicktime_tims_t *tims);
int quicktime_tmax_delete(quicktime_tmax_t *tmax);
int quicktime_tmax_dump(quicktime_tmax_t *tmax);
int quicktime_tmax_init(quicktime_tmax_t *tmax);
int quicktime_tmin_delete(quicktime_tmin_t *tmin);
int quicktime_tmin_dump(quicktime_tmin_t *tmin);
int quicktime_tmin_init(quicktime_tmin_t *tmin);
int quicktime_tpyl_delete(quicktime_tpyl_t *tpyl);
int quicktime_tpyl_dump(quicktime_tpyl_t *tpyl);
int quicktime_tpyl_init(quicktime_tpyl_t *tpyl);
int quicktime_trak_dump(quicktime_trak_t *trak);
int quicktime_trak_duration(quicktime_trak_t *trak, long *duration, long *timescale);
int quicktime_trak_fix_counts(quicktime_t *file, quicktime_trak_t *trak);
int quicktime_trak_shift_offsets(quicktime_trak_t *trak, long offset);
int quicktime_trpy_delete(quicktime_trpy_t *trpy);
int quicktime_trpy_dump(quicktime_trpy_t *trpy);
int quicktime_trpy_init(quicktime_trpy_t *trpy);
int quicktime_udta_delete(quicktime_udta_t *udta);
int quicktime_udta_dump(quicktime_udta_t *udta);
int quicktime_vmhd_delete(quicktime_vmhd_t *vmhd);
int quicktime_vmhd_dump(quicktime_vmhd_t *vmhd);
int quicktime_write_ctts(quicktime_t *file, quicktime_ctts_t *ctts);
int quicktime_write_dimm(quicktime_t *file, quicktime_dimm_t *dimm);
int quicktime_write_dinf(quicktime_t *file, quicktime_dinf_t *dinf);
int quicktime_write_dmax(quicktime_t *file, quicktime_dmax_t *dmax);
int quicktime_write_dmed(quicktime_t *file, quicktime_dmed_t *dmed);
int quicktime_write_dref(quicktime_t *file, quicktime_dref_t *dref);
int quicktime_write_dref_table(quicktime_t *file, quicktime_dref_table_t *table);
int quicktime_write_drep(quicktime_t *file, quicktime_drep_t *drep);
int quicktime_write_elst(quicktime_t *file, quicktime_elst_t *elst, long duration);
int quicktime_write_esds_audio(quicktime_t *file, quicktime_esds_t *esds, int esid);
int quicktime_write_esds_video(quicktime_t *file, quicktime_esds_t *esds, int esid);
int quicktime_write_gmhd(quicktime_t *file, quicktime_gmhd_t *gmhd);
int quicktime_write_gmin(quicktime_t *file, quicktime_gmin_t *gmin);
int quicktime_write_hdlr(quicktime_t *file, quicktime_hdlr_t *hdlr);
int quicktime_write_hinf(quicktime_t *file, quicktime_hinf_t *hinf);
int quicktime_write_hint_hnti(quicktime_t *file, quicktime_hint_hnti_t *hnti);
int quicktime_write_hmhd(quicktime_t *file, quicktime_hmhd_t *hmhd);
int quicktime_write_iods(quicktime_t *file, quicktime_iods_t *iods);
int quicktime_write_matrix(quicktime_t *file, quicktime_matrix_t *matrix);
int quicktime_write_maxr(quicktime_t *file, quicktime_maxr_t *maxr);
int quicktime_write_mdat(quicktime_t *file, quicktime_mdat_t *mdat);
int quicktime_write_mdhd(quicktime_t *file, quicktime_mdhd_t *mdhd);
int quicktime_write_minf(quicktime_t *file, quicktime_minf_t *minf);
int quicktime_write_moov(quicktime_t *file, quicktime_moov_t *moov);
int quicktime_write_mvhd(quicktime_t *file, quicktime_mvhd_t *mvhd);
int quicktime_write_nump(quicktime_t *file, quicktime_nump_t *nump);
int quicktime_write_payt(quicktime_t *file, quicktime_payt_t *payt);
int quicktime_write_pmax(quicktime_t *file, quicktime_pmax_t *pmax);
int quicktime_write_rtp(quicktime_t *file, quicktime_rtp_t *rtp);
int quicktime_write_sdp(quicktime_t *file, quicktime_sdp_t *sdp);
int quicktime_write_smhd(quicktime_t *file, quicktime_smhd_t *smhd);
int quicktime_write_stbl(quicktime_t *file, quicktime_minf_t *minf, quicktime_stbl_t *stbl);
int quicktime_write_stco(quicktime_t *file, quicktime_stco_t *stco);
int quicktime_write_stsc(quicktime_t *file, quicktime_stsc_t *stsc);
int quicktime_write_stsd(quicktime_t *file, quicktime_minf_t *minf, quicktime_stsd_t *stsd);
int quicktime_write_stsd_audio(quicktime_t *file, quicktime_stsd_table_t *table);
int quicktime_write_stsd_hint(quicktime_t *file, quicktime_stsd_table_t *table);
int quicktime_write_stsd_table(quicktime_t *file, quicktime_minf_t *minf, quicktime_stsd_table_t *table);
int quicktime_write_stsd_video(quicktime_t *file, quicktime_stsd_table_t *table);
int quicktime_write_stss(quicktime_t *file, quicktime_stss_t *stss);
int quicktime_write_stsz(quicktime_t *file, quicktime_stsz_t *stsz);
int quicktime_write_stts(quicktime_t *file, quicktime_stts_t *stts);
int quicktime_write_tims(quicktime_t *file, quicktime_tims_t *tims);
int quicktime_write_tmax(quicktime_t *file, quicktime_tmax_t *tmax);
int quicktime_write_tmin(quicktime_t *file, quicktime_tmin_t *tmin);
int quicktime_write_tpyl(quicktime_t *file, quicktime_tpyl_t *tpyl);
int quicktime_write_trak(quicktime_t *file, quicktime_trak_t *trak, long moov_time_scale);
int quicktime_write_trpy(quicktime_t *file, quicktime_trpy_t *trpy);
int quicktime_write_udta(quicktime_t *file, quicktime_udta_t *udta);
int quicktime_write_vmhd(quicktime_t *file, quicktime_vmhd_t *vmhd);
long quicktime_track_end(quicktime_trak_t *trak);
void quicktime_atom_write_footer(quicktime_t *file, quicktime_atom_t *atom);
void quicktime_atom_write_header(quicktime_t *file, quicktime_atom_t *atom, char *text);
int quicktime_edts_dump(quicktime_edts_t *edts);
int quicktime_hint_delete(quicktime_hint_t *hint);
int quicktime_hint_dump(quicktime_hint_t *hint);
int quicktime_hint_init(quicktime_hint_t *hint);
int quicktime_hint_set(quicktime_hint_t *hint, quicktime_trak_t *refTrak);
int quicktime_hint_udta_delete(quicktime_hint_udta_t *hint_udta);
int quicktime_hint_udta_dump(quicktime_hint_udta_t *hint_udta);
int quicktime_hint_udta_init(quicktime_hint_udta_t *hint_udta);
int quicktime_hnti_delete(quicktime_hnti_t *hnti);
int quicktime_hnti_dump(quicktime_hnti_t *hnti);
int quicktime_hnti_init(quicktime_hnti_t *hnti);
int quicktime_mdia_dump(quicktime_mdia_t *mdia);
int quicktime_read_edts(quicktime_t *file, quicktime_edts_t *edts, quicktime_atom_t *edts_atom);
int quicktime_read_hint(quicktime_t *file, quicktime_hint_t *hint, quicktime_atom_t *parent_atom);
int quicktime_read_hint_udta(quicktime_t *file, quicktime_hint_udta_t *hint_udta, quicktime_atom_t *parent_atom);
int quicktime_read_hnti(quicktime_t *file, quicktime_hnti_t *hnti, quicktime_atom_t *parent_atom);
int quicktime_read_mdia(quicktime_t *file, quicktime_mdia_t *mdia, quicktime_atom_t *trak_atom);
int quicktime_read_tkhd(quicktime_t *file, quicktime_tkhd_t *tkhd);
int quicktime_read_tref(quicktime_t *file, quicktime_tref_t *tref, quicktime_atom_t *parent_atom);
int quicktime_read_udta_string(quicktime_t *file, char **string, int *size);
int quicktime_tkhd_delete(quicktime_tkhd_t *tkhd);
int quicktime_tkhd_dump(quicktime_tkhd_t *tkhd);
int quicktime_tkhd_init_hint(quicktime_t *file, quicktime_tkhd_t *tkhd);
int quicktime_tref_dump(quicktime_tref_t *tref);
int quicktime_update_ctts(quicktime_ctts_t *ctts, long sample_offset);
int quicktime_update_stco(quicktime_stco_t *stco, long chunk, long offset);
int quicktime_update_stsc(quicktime_stsc_t *stsc, long chunk, long samples);
int quicktime_update_stss(quicktime_stss_t *stss, long sample);
int quicktime_update_stsz(quicktime_stsz_t *stsz, long sample, long sample_size);
int quicktime_update_stts(quicktime_stts_t *stts, long sample_duration);
int quicktime_write_edts(quicktime_t *file, quicktime_edts_t *edts, long duration);
int quicktime_write_hint(quicktime_t *file, quicktime_hint_t *hint);
int quicktime_write_hint_udta(quicktime_t *file, quicktime_hint_udta_t *hint_udta);
int quicktime_write_hnti(quicktime_t *file, quicktime_hnti_t *hnti);
int quicktime_write_mdia(quicktime_t *file, quicktime_mdia_t *mdia);
int quicktime_write_tkhd(quicktime_t *file, quicktime_tkhd_t *tkhd);
int quicktime_write_tref(quicktime_t *file, quicktime_tref_t *tref);
int quicktime_write_udta_string(quicktime_t *file, char *string, int size);
#ifdef __cplusplus
}
#endif

#endif
